<?php

/**
 * Class Block
 */
class Block
{

    const TYPE_STANDARD    = 1;
    const TYPE_IMAGE       = 2;
    const TYPE_MINIBLOCS   = 3;
    const TYPE_LISTE_PAGE  = 4;
    const TYPE_GROUP_IMAGE = 5;

    const LIEN_BLOCK = 'block_lien';
    const IMAGE_BLOCK = 'block_image';
    const CMS_BLOCK = 'block_page';
    const BOUTIQUE_BLOCK = 'block_boutique';
    const BLOCK_BLOCK = 'block_block';

    const WIDTH_25  = 1;
    const WIDTH_33  = 2;
    const WIDTH_50  = 3;
    const WIDTH_66  = 4;
    const WIDTH_75  = 5;
    const WIDTH_100 = 6;

    const POSITION_HOMEPAGE = 1;

    /**
     * @var null|SiteSQL
     */
    protected $sql = null;

    /**
     * Block constructor.
     */
    public function __construct()
    {
        $this->sql = SiteSQL::getInstance();
    }

    /**
     * @param array $formvars
     * @param string $champs
     * @return null
     */
    public function get($formvars = array(), $champs = "")
    {
        $bId = (!empty($formvars['boutique_id'])) ? $formvars['boutique_id'] : Boutique::getInstance()->id;

        $sql = "SELECT b.block_id, b.block_type_id, b.block_background, b.block_slider";
        $sql .= ", bn.block_nom_titre, bn.block_nom_desc, bn.block_nom_image";
        $sql .= ", bt.block_type_template";
        $sql .= ", bw.block_width_bootstrap_value";

        $sql .= " FROM block b";
        $sql .= " LEFT JOIN block_nom bn ON (bn.block_id = b.block_id AND bn.langue_id = '" . Boutique::getInstance()->getLangue()->id . "')";
        $sql .= " LEFT JOIN block_type bt ON(bt.block_type_id = b.block_type_id)";
        $sql .= " LEFT JOIN block_width bw ON(bw.block_width_id = b.block_width_id)";

        if (empty($formvars["all"])) {
            $sql .= " LEFT JOIN block_boutique bb ON (bb.block_id = b.block_id AND bb.boutique_id = " . $bId . ")";
        }

        $sql .= " LEFT JOIN cms_page cp ON (cp.cms_page_id = b.cms_page_id)";
        $sql .= " WHERE 1";

        if (!empty($formvars["cms_page_id"])) {
            $sql .= " AND b.cms_page_id = '" . $formvars["cms_page_id"] . "'";
        }
        if (!empty($formvars["block_id"])) {
            $sql .= " AND b.block_id = '" . $formvars["block_id"] . "'";
        }
        if (!empty($formvars["cms_page_type_id"])) {
            $sql .= " AND cp.cms_page_type_id = '" . $formvars["cms_page_type_id"] . "'";
        }
        if (!empty($formvars["block_type_id"])) {
            $sql .= " AND b.block_type_id = '" . $formvars["block_type_id"] . "'";
        }
        if (!empty($formvars['block_position_id'])) {
            $sql .= " AND b.block_position_id = '" . $formvars['block_position_id'] . "'";
        }

        if (empty($formvars["all"])) {
            $sql .= " AND bb.block_boutique_statut_id = " . Boutique::STATUT_VISIBLE;
        }
        if (!empty($formvars['indie_block_only'])) {
            $sql .= " AND b.cms_page_id IS NULL AND b.parent_id IS NULL";
        }

        $sql .= " ORDER BY b.block_rang ASC";

        if (empty($champs)) {
            $this->sql->query($sql, SiteSQL::SQL_ALL, SiteSQL::SQL_ASSOC);

            return $this->sql->record;
        } else {
            $this->sql->query($sql, SiteSQL::SQL_INIT, SiteSQL::SQL_ASSOC);

            return $this->sql->record[$champs];
        }
    }

    /**
     * @param array $formvars
     * @return array
     */
    public function getImage($formvars = array())
    {
        $lgId = (!empty($formvars['langue_id'])) ? $formvars['langue_id'] : Boutique::getInstance()->getLangue()->id;

        $sql = "SELECT bi.block_image_id, bi.block_id, bi.block_image_rang";
        $sql .= ", bin.block_image_nom_id, bin.langue_id, bin.block_image_nom_nom, bin.block_image_nom_image";

        $sql .= " FROM block_image bi";
        $sql .= " LEFT JOIN block_image_nom bin ON(bin.block_image_id = bi.block_image_id AND bin.langue_id = '" . $lgId . "')";

        $sql .= " WHERE 1";

        if (!empty($formvars['block_image_id'])) {
            $sql .= " AND bi.block_image_id = '" . $formvars['block_image_id'] . "'";
        }
        if (!empty($formvars['block_id'])) {
            $sql .= " AND bi.block_id = '" . $formvars['block_id'] . "'";
        }

        if (!empty($formvars['order'])) {
            $sql .= " ORDER BY " . $formvars['order'];
        } else {
            $sql .= " ORDER BY bi.block_image_rang ASC";
        }

        return $this->sql->getAll($sql);
    }

    /**
     * @param array $formvars
     * @return array
     */
    public function getPage($formvars = array())
    {
        $lgId = (!empty($formvars['langue_id'])) ? $formvars['langue_id'] : Boutique::getInstance()->getLangue()->id;
        $bId  = (!empty($formvars['boutique_id'])) ? $formvars['boutique_id'] : Boutique::getInstance()->id;

        $sql = "SELECT bp.block_page_id, bp.page_id AS cms_page_id";
        $sql .= ", p.page_rang";
        $sql .= ", bp.block_page_rang, bp.block_page_display_desc";
        $sql .= ", pn.page_nom_nom, pn.page_nom_url, pn.page_nom_image, pn.page_nom_contenu_court, pn.page_nom_image_listing";

        $sql .= " FROM block_page bp ";
        $sql .= " LEFT JOIN cms_page p ON(p.cms_page_id = bp.page_id)";
        $sql .= " LEFT JOIN cms_page_nom pn ON(pn.cms_page_id = p.cms_page_id AND pn.langue_id = '" . $lgId . "')";

        if (empty($formvars['all'])) {
            $sql .= " LEFT JOIN cms_page_boutique pb ON(pb.cms_page_id = p.cms_page_id AND pb.boutique_id = '" . $bId . "')";
        }

        $sql .= " WHERE 1";

        if (!empty($formvars['block_id'])) {
            $sql .= " AND bp.block_id = '" . $formvars['block_id'] . "'";
        }
        if (empty($formvars['all'])) {
            $sql .= " AND pb.cms_page_boutique_statut_id = '" . Boutique::STATUT_VISIBLE . "'";
        }

        if (!empty($formvars['order'])) {
            $sql .= " ORDER BY " . $formvars['order'];
        } else {
            $sql .= " ORDER BY bp.block_page_rang ASC";
        }

        return $this->sql->getAll($sql);
    }

    /**
     * @param array $formvars
     * @return array
     */
    public function getLien($formvars = array())
    {
        $lgId = (!empty($formvars['langue_id'])) ? $formvars['langue_id'] : Boutique::getInstance()->getLangue()->id;

        $sql = "SELECT bl.block_lien_id, bl.block_lien_rang";
        $sql .= ", bln.block_lien_nom_nom, bln.block_lien_nom_url";

        $sql .= " FROM block_lien bl";
        $sql .= " LEFT JOIN block_lien_nom bln ON(bln.block_lien_id = bl.block_lien_id AND bln.langue_id = '" . $lgId . "')";

        $sql .= " WHERE 1";

        if (!empty($formvars['block_id'])) {
            $sql .= " AND bl.block_id = '" . $formvars['block_id'] . "'";
        }

        if (!empty($formvars['order'])) {
            $sql .= " ORDER BY " . $formvars['order'];
        } else {
            $sql .= " ORDER BY bl.block_lien_rang ASC";
        }

        return $this->sql->getAll($sql);
    }

    /**
     * @param array $formvars
     * @return array
     */
    public function getMiniBlock($formvars = array())
    {
        $lgId = (!empty($formvars['langue_id'])) ? $formvars['langue_id'] : Boutique::getInstance()->getLangue()->id;

        $sql = "SELECT b.block_id";
        $sql .= ", bn.block_nom_titre, bn.block_nom_desc, bn.block_nom_image";
        $sql .= ", bw.block_width_bootstrap_value";

        $sql .= " FROM block b";
        $sql .= " LEFT JOIN block_nom bn ON(bn.block_id = b.block_id AND bn.langue_id = '" . $lgId . "')";
        $sql .= " LEFT JOIN block_width bw ON(bw.block_width_id = b.block_width_id)";

        $sql .= " WHERE 1";

        if (!empty($formvars['block_id'])) {
            $sql .= " AND b.parent_id = '" . $formvars['block_id'] . "'";
        } else {
            $sql .= " AND b.parent_id IS NOT NULL";
        }

        if (!empty($formvars['miniblock_id'])) {
            $sql .= " AND b.block_id = '" . $formvars['miniblock_id'] . "'";
        }

        if (!empty($formvars['order'])) {
            $sql .= " ORDER BY " . $formvars['order'];
        } else {
            $sql .= " ORDER BY b.block_rang ASC";
        }

        return $this->sql->getAll($sql);
    }

    /**
     * @param array $blocks
     */
    public function hydrateBlock(&$blocks = array())
    {
        foreach ($blocks as &$block) {
            if (empty($block['block_type_id'])) continue;

            switch ($block['block_type_id']) {
                case self::TYPE_GROUP_IMAGE:
                    $images = $this->getImage(array('block_id' => $block['block_id']));
                    if (!empty($images)) $block['images'] = $images;
                    break;
                case self::TYPE_MINIBLOCS:
                    $miniblocks = $this->getMiniBlock(array('block_id' => $block['block_id']));
                    if (!empty($miniblocks)) $block['miniblocks'] = $miniblocks;
                    break;
                case self::TYPE_LISTE_PAGE:
                    $pages = $this->getPage(array('block_id' => $block['block_id']));
                    if (!empty($pages)) $block['pages'] = $pages;
                    break;
                case self::TYPE_STANDARD:
                    $liens = $this->getLien(array('block_id' => $block['block_id']));
                    if (!empty($liens)) $block['liens'] = $liens;
                    break;
            }
        }
    }

    public function getRows($blocks = array())
    {
        $arrRowTemp = array();
        $incWidth   = 0;
        $nbRow      = 0;

        foreach ($blocks as $block) {
            switch ($block['block_type_id']) {
                case self::TYPE_MINIBLOCS:
                case self::TYPE_LISTE_PAGE:
                    $width = 12;
                    break;
                default:
                    $width = $block['block_width_bootstrap_value'];
                    break;
            }

            $incWidth += $width;

            if ($incWidth > 12) {

                $nbRow++;
                $arrRowTemp[$nbRow][] = $block;
                $incWidth             = $width;

            } elseif ($incWidth == 12) {

                $arrRowTemp[$nbRow][] = $block;
                $nbRow++;
                $incWidth = 0;

            } else {

                $arrRowTemp[$nbRow][] = $block;

            }

        }

        $arrRowFinal = array();

        foreach ($arrRowTemp as $blockRow) {
            $hasBackground = false;
            foreach ($blockRow as $b) {
                if (!empty($b['block_background'])) $hasBackground = true;
            }

            $arrRowFinal[] = array(
                "blocks"           => $blockRow,
                "hasBackground"    => $hasBackground,
                "nbBlocks"         => count($blockRow),
                "firstBlockTypeId" => $blockRow[0]['block_type_id'],
                "hasBlockContact" => $this->rowHasType(self::TYPE_CONTACT, $blockRow)
            );

        }

        return $arrRowFinal;
    }

    /**
     * @param $typeId
     * @param array $blocks
     * @return bool
     */
    public function rowHasType($typeId, $blocks = array())
    {
        if (empty($typeId)) return false;

        if (empty($blocks)) return false;

        foreach ($blocks as $block) {
            if ($block['block_type_id'] == $typeId) return true;
        }

        return false;

    }


}